@@ -7,7 +7,7 @@
    Copyright (C) 2004, 2005, 2006
    ViewPlus Technologies, Inc. www.viewplus.com
    and
-   Abilitiessoft, Inc. www.abilitiessoft.com
+   Abilitiessoft, Inc. www.abilitiessoft.org
    All rights reserved
 
    This file is free software; you can redistribute it and/or modify it
@@ -26,7 +26,7 @@
    the Free Software Foundation, 51 Franklin Street, Fifth Floor,
    Boston, MA 02110-1301, USA.
 
-   Maintained by John J. Boyer john.boyer@abilitiessoft.com
+   Maintained by John J. Boyer john.boyer@abilitiessoft.org
    */
 
 #ifndef liblouisutdml_h
@@ -76,6 +76,7 @@ typedef struct
   int keep_with_next;
   int dont_split;
   int orphan_control;
+  int widow_control;
   int first_line_indent;	/* At true margin if negative */
   char *translation_table;
   int skip_number_lines;	/*Don't write on lines with page numbers */
@@ -87,6 +88,8 @@ typedef struct
   int newline_after;
   int runningHead;
   sem_act emphasis;
+  char topBoxline[1];
+  char bottomBoxline[1];
   char name[1];
 } StyleType;
 
@@ -185,6 +188,8 @@ typedef struct
   FormatFor orig_format_for;
   int contents;
   int has_contentsheader;
+  int endnotes;
+  int endnote_stage;
   unsigned int mode;
   unsigned int orig_mode;
   unsigned int config_mode;
@@ -278,6 +283,7 @@ typedef struct
   const char *compbrl_table_name;
   const char *mathtext_table_name;
   const char *mathexpr_table_name;
+  const char *pagenum_table_name;
   const char *edit_table_name;
   const char *main_braille_table;
   const char *semantic_files;
@@ -345,6 +351,13 @@ int start_heading (sem_act action, widechar * translatedBuffer, int
 		   translatedLength);
 int finish_heading (sem_act action);
 int make_contents ();
+int initialize_endnotes();
+int make_endnotes();
+int link_endnote(xmlNode* node);
+void set_notes_header();
+void set_notes_description();
+int start_endnote();
+int finish_endnote(xmlNode* node);
 void do_reverse (xmlNode * node);
 int do_boxline (xmlNode * node);
 void do_pagebreak (xmlNode *node);
@@ -412,4 +425,8 @@ unsigned char *get_sem_name (xmlNode *node);
 void set_runninghead_string (widechar *chars, int length);
 void set_footer_string (widechar *chars, int length);
 void do_utdxxxximg (xmlNode *node);
+
+void logWidecharBuf(logLevels level, const char *msg, const widechar *wbuf, int wlen);
+
+// void logMessage(logLevels level, const char *format, ...);
 #endif /*louisutdml_h */
