@@ -90,9 +90,9 @@ configureError (FileInfo * nested, char *format, ...)
 #endif
   va_end (arguments);
   if (nested)
-    lou_logPrint ("%s:%d: %s", nested->fileName, nested->lineNumber, buffer);
+    logMessage (LOG_ERROR, "%s:%d: %s", nested->fileName, nested->lineNumber, buffer);
   else
-    lou_logPrint ("%s", buffer);
+    logMessage (LOG_ERROR, "%s", buffer);
   errorCount++;
 }
 
@@ -711,8 +711,10 @@ compileConfig (FileInfo * nested)
     "55",
     "mode",
     "56",
-// available    "57",
-// available     "58",
+    "pageNumberTable",
+    "57",
+	"Endnotes",
+	"58",
 // available    "59",
     "macro",
     "60",
@@ -1064,8 +1066,18 @@ compileConfig (FileInfo * nested)
 	  if ((k = orValues (nested, configModes)) != NOTFOUND)
 	    ud->config_mode = k;
 	  break;
-// available	case 57:
-// available	case 58:
+	case 57:
+	  ud->pagenum_table_name = findTable (nested);
+	  if (ud->pagenum_table_name == NULL)
+	  {
+	  configureError (nested, "invalid pageNumberTable");
+	  return 0;
+	  }
+	  break;
+	case 58:
+	  if ((k = checkValues (nested, yesNo)) != NOTFOUND)
+	    ud->endnotes = k;
+	  break;
 // available	case 59:
 	case 60:
 	  new_macro (nested->value, nested->value2);
@@ -1109,6 +1121,10 @@ compileConfig (FileInfo * nested)
 	      "18",
 	      "emphasis",
 	      "19",
+              "topBoxline",
+              "20",
+              "bottomBoxline",
+              "21",
 	      NULL
 	    };
 	    static const char *formats[] = {
@@ -1239,6 +1255,12 @@ compileConfig (FileInfo * nested)
 		  nested->value);
 		  }
 		  break;
+                  case 20:
+                    memcpy(style->topBoxline, nested->value, 1);
+                    break;
+                  case 21:
+                    memcpy(style->bottomBoxline, nested->value, 1);
+                    break;
 		  default:
 		    configureError (nested, "Program error in readconfig.c");
 		    continue;
@@ -1273,7 +1295,7 @@ initConfigFiles (const char *firstConfigFile, char *fileName, const
     {
       strcpy ((char *) ud->typeform, lbu_getWriteablePath ());
       strcat ((char *) ud->typeform, logFileName);
-      lou_logFile ((char *) ud->typeform);
+      lbu_logFile ((char *) ud->typeform);
     }
   if (!config_compileSettings ("liblouisutdml.ini"))
     return 0;
@@ -1291,6 +1313,7 @@ read_configuration_file (const char *configFileList, const char
   char subFile[MAXNAMELEN];
   int listLength;
   int currentListPos = 0;
+  logMessage(LOG_INFO, "Begin read_configuration_file");
   errorCount = 0;
   fatalErrorCount = 0;
   /*Process logFileName later, after writeablePath is set */
@@ -1355,7 +1378,7 @@ read_configuration_file (const char *configFileList, const char
     {
       strcpy ((char *) ud->typeform, lbu_getWriteablePath ());
       strcat ((char *) ud->typeform, logFileName);
-      lou_logFile ((char *) ud->typeform);
+      lbu_logFile ((char *) ud->typeform);
     }
       if (!(config_compileSettings ("liblouisutdml.ini")))
 	return 0;
@@ -1439,8 +1462,8 @@ read_configuration_file (const char *configFileList, const char
     {
       if (ud->page_right <= 0 || ud->page_bottom <= 0)
 	{
-	  lou_logPrint
-	    ("For UTDML paper witdth and paper height must be specified.");
+	  logMessage
+	    (LOG_ERROR, "For UTDML paper witdth and paper height must be specified.");
 	  lbu_free ();
 	  return 0;
 	}
@@ -1450,5 +1473,6 @@ read_configuration_file (const char *configFileList, const char
       ud->back_text = textDevice;
       ud->back_line_length = 70;
     }
+  logMessage(LOG_INFO, "Finish read_configuration_file");
   return 1;
 }
