@@ -104,9 +104,9 @@ semanticError (FileInfo * nested, char *format, ...)
 #endif
   va_end (arguments);
   if (nested)
-    lou_logPrint ("%s:%d: %s", nested->fileName, nested->lineNumber, buffer);
+    logMessage (LOG_ERROR, "%s:%d: %s", nested->fileName, nested->lineNumber, buffer);
   else
-    lou_logPrint ("%s", buffer);
+    logMessage (LOG_ERROR, "%s", buffer);
   errorCount++;
 }
 
@@ -868,6 +868,7 @@ sem_compileFile (const char *fileName)
   FileInfo nested;
   char completePath[MAXNAMELEN];
   int haveAppended = 0;
+  logMessage(LOG_INFO, "Begin sem_compileFile: fileName=%s", fileName);
   if (!*fileName)
     return 1;			/*Probably run with defaults */
   if (strncmp (fileName, "appended_", 9) == 0)
@@ -889,6 +890,7 @@ sem_compileFile (const char *fileName)
   memset (&nested, 0, sizeof (nested));
   nested.fileName = fileName;
   nested.unedited = 1;
+  logMessage(LOG_INFO, "Loading semantic action file: %s", completePath);
   if ((nested.in = fopen ((char *) completePath, "rb")))
     {
       while (getALine (&nested))
@@ -908,6 +910,7 @@ sem_compileFile (const char *fileName)
       return 0;
     }
   numEntries += nested.numEntries;
+  logMessage(LOG_INFO, "Finish sem_compileFile");
   return 1;
 }
 
@@ -1077,7 +1080,7 @@ printXpathNodes (xmlNodeSetPtr nodes)
   int size;
   int i;
   size = (nodes) ? nodes->nodeNr : 0;
-  semanticError (NULL, "Result (%d nodes):", size);
+  logMessage (LOG_INFO, "Result (%d nodes):", size);
   for (i = 0; i < size; ++i)
     {
       if (nodes->nodeTab[i]->type == XML_NAMESPACE_DECL)
@@ -1087,13 +1090,13 @@ printXpathNodes (xmlNodeSetPtr nodes)
 	  cur = (xmlNodePtr) ns->next;
 	  if (cur->ns)
 	    {
-	      semanticError (NULL,
+	      logMessage (LOG_INFO,
 			     "= namespace \"%s\"=\"%s\" for node %s:%s",
 			     ns->prefix, ns->href, cur->ns->href, cur->name);
 	    }
 	  else
 	    {
-	      semanticError (NULL, "= namespace \"%s\"=\"%s\" for node %s",
+	      logMessage (LOG_INFO, "= namespace \"%s\"=\"%s\" for node %s",
 			     ns->prefix, ns->href, cur->name);
 	    }
 	}
@@ -1102,18 +1105,18 @@ printXpathNodes (xmlNodeSetPtr nodes)
 	  cur = nodes->nodeTab[i];
 	  if (cur->ns)
 	    {
-	      semanticError (NULL, "= element node \"%s:%s\"",
+	      logMessage (LOG_INFO, "= element node \"%s:%s\"",
 			     cur->ns->href, cur->name);
 	    }
 	  else
 	    {
-	      semanticError (NULL, "= element node \"%s\"", cur->name);
+	      logMessage (LOG_INFO, "= element node \"%s\"", cur->name);
 	    }
 	}
       else
 	{
 	  cur = nodes->nodeTab[i];
-	  semanticError (NULL, "= node \"%s\": type %d", cur->name,
+	  logMessage (LOG_INFO, "= node \"%s\": type %d", cur->name,
 			 cur->type);
 	}
     }
@@ -1225,9 +1228,15 @@ get_sem_attr (xmlNode * node)
 {
   HashEntry *nodeEntry = (HashEntry *) node->_private;
   if (nodeEntry != NULL)
+  {
+    logMessage(LOG_DEBUG, "Node %s has nodeEntry", (const char *)node->name);
     return nodeEntry->semNum;
+  }
   else
+  {
+    logMessage(LOG_DEBUG, "Node %s has no nodeEntry", (const char *)node->name);
     return no;
+  }
 }
 
 unsigned char *
@@ -1410,10 +1419,10 @@ append_new_entries ()
     }
   fclose (semOut);
   if (haveSemanticFile)
-    lou_logPrint ("%d new entries appended to '%s%s'.", numEntries,
+    logMessage (LOG_WARN, "%d new entries appended to '%s%s'.", numEntries,
 		  filePrefix, firstFileName);
   else
-    lou_logPrint ("%d entries written to new semantic-action file '%s%s'.",
+    logMessage (LOG_WARN, "%d entries written to new semantic-action file '%s%s'.",
 		  numEntries, filePrefix, firstFileName);
   moreEntries = 0;
 }
@@ -1508,7 +1517,7 @@ macroError (char *format, ...)
   vsnprintf (buffer, sizeof (buffer), format, arguments);
 #endif
   va_end (arguments);
-  lou_logPrint ("Macro %s: %s", macroName, buffer);
+  logMessage (LOG_ERROR, "Macro %s: %s", macroName, buffer);
 }
 
 char *
@@ -1616,7 +1625,7 @@ doSemanticActions ()
       ud->main_braille_table = ud->contracted_table_name;
       if (!lou_getTable (ud->main_braille_table))
 	{
-	  lou_logPrint ("Cannot open main table %s", ud->main_braille_table);
+	  logMessage (LOG_ERROR, "Cannot open main table %s", ud->main_braille_table);
 	  return 0;
 	}
       break;
@@ -1630,9 +1639,6 @@ doSemanticActions ()
 	break;
       insert_linkOrTarget (macroNode, 1);
       break;
-    case boxline:
-      do_boxline (macroNode);
-      break;
     case blankline:
       do_blankline ();
       break;
@@ -1794,6 +1800,7 @@ executeMacro ()
 	end_style ();
       macro = NULL;
     }
+  return 1;
 }
 
 int
