@@ -7,7 +7,7 @@
    Copyright (C) 2004, 2005, 2006
    ViewPlus Technologies, Inc. www.viewplus.com
    and
-   Abilitiessoft, Inc. www.abilitiessoft.com
+   Abilitiessoft, Inc. www.abilitiessoft.org
    All rights reserved
 
    This file is free software; you can redistribute it and/or modify it
@@ -26,7 +26,7 @@
    the Free Software Foundation, 51 Franklin Street, Fifth Floor,
    Boston, MA 02110-1301, USA.
 
-   Maintained by John J. Boyer john.boyer@abilitiessoft.com
+   Maintained by John J. Boyer john.boyer@abilitiessoft.org
    */
 
 #include <stdio.h>
@@ -35,7 +35,7 @@
 
 extern void widecharcpy (widechar * to, const widechar * from, int length);
 extern void widestrcpy (widechar * to, const widechar * from);
-extern void unsignedcharcpy (char * to, const char * from, int length);
+extern void unsignedcharcpy (char *to, const char *from, int length);
 extern void save_translated_buffer (void);
 extern void restore_translated_buffer (void);
 extern void contents_save_state (void);
@@ -50,6 +50,9 @@ static int keep_with_previous_status = 0;
 static int orphan_control = 0;
 static int orphan_control_pos = 0;
 static int orphan_control_status = 0;
+static int widow_control = 0;
+static int widow_control_pos = 0;
+static int widow_control_status = 0;
 static int saved_text_length;
 static int saved_sync_text_length;
 static int saved_translated_length;
@@ -189,11 +192,13 @@ transcribe_paragraph (xmlNode * node, int action)
   int keep_with_next_this = 0;
   int keep_with_previous_this = 0;
   int orphan_control_this = 0;
+  int widow_control_this = 0;
   StyleType *style;
   int haveMacro = 0;
   xmlNode *child;
   int branchCount = 0;
   int i;
+  logMessage(LOG_DEBUG, "Begin transcribe_paragraph");
   if (node == NULL)
     return 0;
   if (ud->top == 0)
@@ -203,6 +208,8 @@ transcribe_paragraph (xmlNode * node, int action)
   switch (ud->stack[ud->top])
     {
     case no:
+      if (ud->format_for == utd)
+	break;
       if (ud->text_length > 0 && ud->text_length < MAX_LENGTH &&
 	  ud->text_buffer[ud->text_length - 1] > 32)
 	ud->text_buffer[ud->text_length++] = 32;
@@ -220,12 +227,12 @@ transcribe_paragraph (xmlNode * node, int action)
       ud->main_braille_table = ud->contracted_table_name;
       if (!lou_getTable (ud->main_braille_table))
 	{
-	  lou_logPrint ("Cannot open main table %s", ud->main_braille_table);
+	  logMessage (LOG_ERROR, "Cannot open main table %s", ud->main_braille_table);
 	  return 0;
 	}
-    if (node->children == NULL)
-      return 1;
-    break;
+      if (node->children == NULL)
+	return 1;
+      break;
     case htmllink:
       if (ud->format_for != browser)
 	break;
@@ -240,44 +247,39 @@ transcribe_paragraph (xmlNode * node, int action)
       if (action != 0)
 	pop_sem_stack ();
       return 1;
-    case boxline:
-      do_boxline (node);
-    if (node->children == NULL)
-      return 1;
-    break;
     case pagebreak:
       do_pagebreak (node);
       return 1;
     case attrtotext:
       do_attrtotext (node);
-    if (node->children == NULL)
-      return 1;
-    break;
+      if (node->children == NULL)
+	return 1;
+      break;
     case blankline:
       do_blankline ();
-    if (node->children == NULL)
-      return 1;
-    break;
+      if (node->children == NULL)
+	return 1;
+      break;
     case linespacing:
       do_linespacing (node);
-    if (node->children == NULL)
-      return 1;
-    break;
+      if (node->children == NULL)
+	return 1;
+      break;
     case softreturn:
       do_softreturn ();
-    if (node->children == NULL)
-      return 1;
-    break;
+      if (node->children == NULL)
+	return 1;
+      break;
     case newpage:
       do_newpage ();
-    if (node->children == NULL)
-      return 1;
-    break;
+      if (node->children == NULL)
+	return 1;
+      break;
     case righthandpage:
       do_righthandpage ();
-    if (node->children == NULL)
-      return 1;
-    break;
+      if (node->children == NULL)
+	return 1;
+      break;
     case code:
       transcribe_computerCode (node, 0);
       if (action != 0)
@@ -311,9 +313,19 @@ transcribe_paragraph (xmlNode * node, int action)
       return 1;
     case changetable:
       change_table (node);
-    if (node->children == NULL)
-      return 1;
-    break;
+            //if (node->children == NULL)			////?? removed, as change_table() cycles through all children anyway, so if changetable has child nodes (ie. <changetable><xyz>Text</xyz><changetable>) then produces duplicate outputs.
+			return 1;
+      break;
+	case noteref:							////
+		if(ud->endnotes)
+			start_endnote(node);
+		else
+		{
+			if(action!=0)
+				pop_sem_stack();
+			return 1;
+		}
+		break;
     case pagenum:
       do_pagenum ();
       break;
@@ -323,10 +335,20 @@ transcribe_paragraph (xmlNode * node, int action)
   if (is_macro (node))
     {
       haveMacro = 1;
+      logMessage(LOG_DEBUG, "Node has macro");
       start_macro (node);
     }
   else if ((style = is_style (node)) != NULL)
-    start_style (style, node);
+    {
+      if (node->children == NULL)
+	{
+	  if (action != 0)
+	    pop_sem_stack ();
+	  return 0;
+	}
+      logMessage(LOG_DEBUG, "Node has style");
+      start_style (style, node);
+    }
   child = node->children;
   while (child)
     {
@@ -360,10 +382,13 @@ transcribe_paragraph (xmlNode * node, int action)
 			    dont_split_this = 1;
 			  else if (style_this->orphan_control > 1)
 			    orphan_control_this = style_this->orphan_control;
+			  else if (style_this->widow_control > 1)
+				widow_control_this = style_this->widow_control;
+
 			}
 		      keep_with_next_this = style_this->keep_with_next;
 		    }
-		  if (dont_split_this || orphan_control_this)
+		  if (dont_split_this || orphan_control_this || widow_control_this)
 		    {
 		      if (!ud->outbuf3_enabled)
 			{
@@ -389,6 +414,11 @@ transcribe_paragraph (xmlNode * node, int action)
 		      orphan_control = orphan_control_this;
 		      orphan_control_pos = ud->lines_length;
 		    }
+		  if (widow_control_this)
+		  {
+			widow_control = widow_control_this;
+			widow_control_pos = ud->lines_length;
+		  }
 		}
 	    }
 	  if (strcmp (child->name, "brl") != 0)
@@ -421,7 +451,8 @@ transcribe_paragraph (xmlNode * node, int action)
 	    {
 	      if (dont_split_status >= 0 &&
 		  keep_with_previous_status >= 0
-		  && orphan_control_status >= 0)
+		  && orphan_control_status >= 0
+		  && widow_control_status >=0)
 		{
 		  if (keep_with_previous)
 		    {
@@ -472,9 +503,32 @@ transcribe_paragraph (xmlNode * node, int action)
 		      if (i == orphan_control)
 			orphan_control_status = 1;
 		    }
+			
+		  if (widow_control)
+		    {
+		      if (widow_control_this)
+				widow_control_status = 1;
+		      else
+				widow_control_status = 0;
+		      i = ud->lines_length-widow_control_pos-1;
+		      while (i > ud->lines_length-widow_control && i > 0)
+			  {
+				if (ud->lines_pagenum[widow_control_pos + i] >
+			      ud->lines_pagenum[widow_control_pos + i - 1])
+			    {
+			      if (!ud->lines_newpage[i])
+					widow_control_status = -1;
+			      break;
+			    }
+				i--;
+			  }
+		      if (i == ud->lines_length-widow_control)
+				widow_control_status = 1;
+		    }
 		}
 	      if (dont_split_status < 0 ||
-		  keep_with_previous_status < 0 || orphan_control_status < 0)
+		  keep_with_previous_status < 0 || orphan_control_status < 0 ||
+		  widow_control_status < 0)
 		{
 		  if (state_saved)
 		    {
@@ -485,6 +539,8 @@ transcribe_paragraph (xmlNode * node, int action)
 		      keep_with_previous_status = 0;
 		      orphan_control = 0;
 		      orphan_control_status = 0;
+			  widow_control = 0;
+			  widow_control_status = 0;	
 		      restoreState ();
 		      child = saved_child;
 		      branchCount = saved_branchCount;
@@ -511,9 +567,14 @@ transcribe_paragraph (xmlNode * node, int action)
 		  orphan_control = 0;
 		  orphan_control_status = 0;
 		}
-	      if ((!dont_split && !keep_with_previous &&
-		   !keep_with_next && !orphan_control) || (!child
-							   && state_saved))
+		if(widow_control_status > 0)
+		{
+			widow_control = 0;
+			widow_control_status = 0;
+		}
+	    if ((!dont_split && !keep_with_previous &&
+		   !keep_with_next && !orphan_control 
+		   &&!widow_control) ||(!child && state_saved))
 		{
 		  write_buffer (3, 0);
 		  ud->outbuf3_enabled = 0;
@@ -528,6 +589,8 @@ transcribe_paragraph (xmlNode * node, int action)
 	    keep_with_previous = 0;
 	  if (orphan_control_this)
 	    orphan_control = 0;
+	  if(widow_control_this)
+		widow_control = 0;
 	}
     }
   insert_code (node, branchCount);
@@ -549,6 +612,10 @@ transcribe_paragraph (xmlNode * node, int action)
 	set_footer_string (ud->translated_buffer, ud->translated_length);
 	ud->translated_length = 0;
 	break;
+	case noteref:
+		if(ud->endnotes)
+			finish_endnote(node);
+		break;
       default:
 	break;
       }
@@ -559,5 +626,6 @@ transcribe_paragraph (xmlNode * node, int action)
       insert_translation (ud->main_braille_table);
       write_paragraph (para, NULL);
     }
+  logMessage(LOG_DEBUG, "Finished transcribe_paragraph");
   return 1;
 }
